@@ -1,46 +1,26 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2011, Red Hat, Inc., and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 package org.jboss.as.ejb3.timerservice;
 
 import java.io.Serializable;
 import java.util.Date;
 import java.util.concurrent.Semaphore;
-import javax.ejb.EJBException;
-import javax.ejb.ScheduleExpression;
-import javax.ejb.Timer;
-import javax.ejb.TimerHandle;
+import jakarta.ejb.EJBException;
+import jakarta.ejb.ScheduleExpression;
+import jakarta.ejb.TimerHandle;
 
-import org.jboss.as.ejb3.component.allowedmethods.AllowedMethodsInformation;
-import org.jboss.as.ejb3.component.allowedmethods.MethodType;
 import org.jboss.as.ejb3.logging.EjbLogger;
+import org.jboss.as.ejb3.timerservice.spi.ManagedTimer;
 import org.jboss.as.ejb3.timerservice.spi.TimedObjectInvoker;
 
 /**
- * Implementation of Enterprise Beans 3.1 {@link Timer}
+ * Local implementation of {@link ManagedTimer}.
  *
  * @author <a href="mailto:cdewolf@redhat.com">Carlo de Wolf</a>
- * @version $Revision: $
  */
-public class TimerImpl implements Timer {
+public class TimerImpl extends AbstractManagedTimer {
     /**
      * The output format with the cached final values, after the first toString() invocation
      */
@@ -56,7 +36,7 @@ public class TimerImpl implements Timer {
     protected volatile TimerState timerState;
 
     /**
-     * The {@link javax.ejb.TimerService} through which this timer was created
+     * The {@link jakarta.ejb.TimerService} through which this timer was created
      */
     protected final TimerServiceImpl timerService;
 
@@ -68,19 +48,13 @@ public class TimerImpl implements Timer {
     /**
      * The info which was passed while creating the timer.
      */
-    protected final Serializable info;
+    protected Serializable info;
 
     /**
      * Indicates whether the timer is persistent
      */
     protected final boolean persistent;
 
-    /**
-     * A {@link javax.ejb.TimerHandle} for this timer, not assigned till
-     * {@code getHandle()} is called.
-     */
-    protected TimerHandleImpl handle;
-
     /**
      * The initial (first) expiry date of this timer
      */
@@ -91,11 +65,6 @@ public class TimerImpl implements Timer {
      */
     protected final long intervalDuration;
 
-    /**
-     * If this is an entity bean then this is the primary key
-     */
-    protected final Object primaryKey;
-
     /**
      * Next expiry date of this timer
      */
@@ -132,6 +101,7 @@ public class TimerImpl implements Timer {
      * @param service          The timer service through which this timer was created
      */
     protected TimerImpl(Builder builder, TimerServiceImpl service) {
+        super(builder.timedObjectId, builder.id);
         assert builder.id != null : "id is null";
 
         this.id = builder.id;
@@ -146,8 +116,6 @@ public class TimerImpl implements Timer {
             this.nextExpiration = builder.nextDate;
         }
         this.previousRun = builder.previousRun;
-        this.primaryKey = builder.primaryKey;
-
         this.timerState = builder.timerState;
         this.timerService = service;
         this.timedObjectInvoker = service.getInvoker();
@@ -170,6 +138,7 @@ public class TimerImpl implements Timer {
      *
      * @return
      */
+    @Override
     public String getId() {
         return this.id;
     }
@@ -180,7 +149,7 @@ public class TimerImpl implements Timer {
     @Override
     public boolean isCalendarTimer() throws IllegalStateException, EJBException {
         // first check whether this timer has expired or cancelled
-        this.assertTimerState();
+        this.validateInvocationContext();
 
         return false;
     }
@@ -191,19 +160,13 @@ public class TimerImpl implements Timer {
     @Override
     public TimerHandle getHandle() throws IllegalStateException, EJBException {
         // make sure it's in correct state
-        this.assertTimerState();
+        this.validateInvocationContext();
 
         // for non-persistent timers throws an exception (mandated by Enterprise Beans 3 spec)
         if (!persistent) {
             throw EjbLogger.EJB3_TIMER_LOGGER.invalidTimerHandlersForPersistentTimers("Enterprise Beans 3.1 Spec 18.2.6");
         }
-
-        synchronized (this) {
-            if (handle == null) {
-                handle = new TimerHandleImpl(id, timedObjectId, timerService);
-            }
-        }
-        return handle;
+        return new TimerHandleImpl(this, timedObjectInvoker.getComponent());
     }
 
     /**
@@ -212,7 +175,7 @@ public class TimerImpl implements Timer {
     @Override
     public boolean isPersistent() throws IllegalStateException, EJBException {
         // make sure the call is allowed in the current timer state
-        this.assertTimerState();
+        this.validateInvocationContext();
         return this.persistent;
     }
 
@@ -224,19 +187,42 @@ public class TimerImpl implements Timer {
     @Override
     public Serializable getInfo() throws IllegalStateException, EJBException {
         // make sure this call is allowed
-        this.assertTimerState();
-        return this.info;
+        this.validateInvocationContext();
+        return getTimerInfo();
     }
 
     /**
      * This method is similar to {@link #getInfo()}, except that this method does <i>not</i> check the timer state
-     * and hence does <i>not</i> throw either {@link IllegalStateException} or {@link javax.ejb.NoSuchObjectLocalException}
-     * or {@link javax.ejb.EJBException}.
+     * and hence does <i>not</i> throw either {@link IllegalStateException} or {@link jakarta.ejb.NoSuchObjectLocalException}
+     * or {@link jakarta.ejb.EJBException}.
      *
-     * @return
+     * @return the timer info; if not available in-memory, retrieve it from persistence
      */
     public Serializable getTimerInfo() {
-        return this.info;
+        if (info != Object.class) {
+            return info;
+        }
+        return timerService.getPersistedTimerInfo(this);
+    }
+
+    /**
+     * Obtains the timer info cached in memory, without checking the persistent store.
+     *
+     * @return the cached timer info
+     */
+    public Serializable getCachedTimerInfo() {
+        return info;
+    }
+
+    /**
+     * Sets the timer info to a new value.
+     * The purpose of this method is for {@code DatabaseTimerPersistence} to reset
+     * the cached timer info. It should not be used for other purposes.
+     *
+     * @param newInfo the new timer info, typically null or an empty holder value
+     */
+    public void setCachedTimerInfo(final Serializable newInfo) {
+        info = newInfo;
     }
 
     /**
@@ -247,7 +233,7 @@ public class TimerImpl implements Timer {
     @Override
     public Date getNextTimeout() throws IllegalStateException, EJBException {
         // first check the validity of the timer state
-        this.assertTimerState();
+        this.validateInvocationContext();
         if (this.nextExpiration == null) {
             throw EjbLogger.EJB3_TIMER_LOGGER.noMoreTimeoutForTimer(this);
         }
@@ -256,8 +242,8 @@ public class TimerImpl implements Timer {
 
     /**
      * This method is similar to {@link #getNextTimeout()}, except that this method does <i>not</i> check the timer state
-     * and hence does <i>not</i> throw either {@link IllegalStateException} or {@link javax.ejb.NoSuchObjectLocalException}
-     * or {@link javax.ejb.EJBException}.
+     * and hence does <i>not</i> throw either {@link IllegalStateException} or {@link jakarta.ejb.NoSuchObjectLocalException}
+     * or {@link jakarta.ejb.EJBException}.
      *
      * @return
      */
@@ -282,7 +268,7 @@ public class TimerImpl implements Timer {
      */
     @Override
     public ScheduleExpression getSchedule() throws IllegalStateException, EJBException {
-        this.assertTimerState();
+        this.validateInvocationContext();
         throw EjbLogger.EJB3_TIMER_LOGGER.invalidTimerNotCalendarBaseTimer(this);
     }
 
@@ -294,7 +280,7 @@ public class TimerImpl implements Timer {
         // TODO: Rethink this implementation
 
         // first check the validity of the timer state
-        this.assertTimerState();
+        this.validateInvocationContext();
         if (this.nextExpiration == null) {
             throw EjbLogger.EJB3_TIMER_LOGGER.noMoreTimeoutForTimer(this);
         }
@@ -318,15 +304,6 @@ public class TimerImpl implements Timer {
         return false;
     }
 
-    /**
-     * Cancels any scheduled timer task for this timer
-     */
-    protected void cancelTimeout() {
-        // delegate to the timerservice, so that it can cancel any scheduled Future
-        // for this timer
-        this.timerService.cancelTimeout(this);
-    }
-
     /**
      * Returns the initial (first) timeout date of this timer
      *
@@ -380,8 +357,9 @@ public class TimerImpl implements Timer {
      *
      * @return
      */
+    @Override
     public boolean isActive() {
-        return timerService.isStarted() && !isCanceled() && !isExpired() && (timerService.isScheduled(getId()) || timerState == TimerState.CREATED);
+        return timerService.isStarted() && !isCanceled() && !isExpired() && (timerState == TimerState.CREATED || timerService.isScheduled(getId()));
     }
 
     /**
@@ -389,6 +367,7 @@ public class TimerImpl implements Timer {
      *
      * @return
      */
+    @Override
     public boolean isCanceled() {
         return timerState == TimerState.CANCELED;
     }
@@ -398,6 +377,7 @@ public class TimerImpl implements Timer {
      *
      * @return
      */
+    @Override
     public boolean isExpired() {
         return timerState == TimerState.EXPIRED;
     }
@@ -447,23 +427,6 @@ public class TimerImpl implements Timer {
         return executingThread;
     }
 
-    /**
-     * Asserts that the timer is <i>not</i> in any of the following states:
-     * <ul>
-     * <li>{@link TimerState#CANCELED}</li>
-     * <li>{@link TimerState#EXPIRED}</li>
-     * </ul>
-     *
-     * @throws javax.ejb.NoSuchObjectLocalException if the txtimer was canceled or has expired
-     */
-    protected void assertTimerState() {
-        if (timerState == TimerState.EXPIRED)
-            throw EjbLogger.EJB3_TIMER_LOGGER.timerHasExpired(id);
-        if (timerState == TimerState.CANCELED)
-            throw EjbLogger.EJB3_TIMER_LOGGER.timerWasCanceled(id);
-        AllowedMethodsInformation.checkAllowed(MethodType.TIMER_SERVICE_METHOD);
-    }
-
     /**
      * Sets the state and timer task executing thread of this timer
      *
@@ -476,6 +439,11 @@ public class TimerImpl implements Timer {
         this.executingThread = thread;
     }
 
+    @Override
+    public void activate() {
+        this.scheduleTimeout(true);
+    }
+
     /**
      * Suspends any currently scheduled task for this timer
      * <p>
@@ -487,9 +455,12 @@ public class TimerImpl implements Timer {
      */
     // TODO: Revisit this method, we probably don't need this any more.
     // In terms of implementation, this is just equivalent to cancelTimeout() method
+    @Override
     public void suspend() {
         // cancel any scheduled timer task (Future) for this timer
-        this.cancelTimeout();
+        // delegate to the timerservice, so that it can cancel any scheduled Future
+        // for this timer
+        this.timerService.cancelTimeout(this);
     }
 
     /**
@@ -499,7 +470,8 @@ public class TimerImpl implements Timer {
      * protection against overlapping events when running it. This is the expected behaviour, as otherwise the semantics
      * of dealing with concurrent execution is complex and kinda weird.
      */
-    public void invokeOneOff() throws Exception {
+    @Override
+    public void invoke() throws Exception {
         this.getTimerTask().invokeBeanMethod(this);
     }
     /**
@@ -518,8 +490,8 @@ public class TimerImpl implements Timer {
      * @return
      * @see TimerTask
      */
-    protected TimerTask<?> getTimerTask() {
-        return new TimerTask<TimerImpl>(this);
+    protected TimerTask getTimerTask() {
+        return new TimerTask(this);
     }
 
     public void lock() throws InterruptedException {
@@ -530,22 +502,6 @@ public class TimerImpl implements Timer {
         inUseLock.release();
     }
 
-    @Override
-    public boolean equals(final Object o) {
-        if (this == o) return true;
-        if (o == null || this.getClass() != o.getClass()) return false;
-
-        final TimerImpl otherTimer = (TimerImpl) o;
-
-        if (!id.equals(otherTimer.id)) return false;
-        return timedObjectId.equals(otherTimer.timedObjectId);
-    }
-
-    @Override
-    public int hashCode() {
-        return id.hashCode();
-    }
-
     /**
      * A nice formatted string output for this timer
      * {@inheritDoc}
@@ -585,10 +541,6 @@ public class TimerImpl implements Timer {
         return sb.toString();
    }
 
-    public Object getPrimaryKey() {
-        return primaryKey;
-    }
-
     public static Builder builder() {
         return new Builder();
     }
@@ -601,7 +553,6 @@ public class TimerImpl implements Timer {
         protected Date nextDate;
         protected Date previousRun;
         protected Serializable info;
-        protected Object primaryKey;
         protected TimerState timerState;
         protected boolean persistent;
         protected boolean newTimer;
@@ -641,11 +592,6 @@ public class TimerImpl implements Timer {
             return this;
         }
 
-        public Builder setPrimaryKey(final Object primaryKey) {
-            this.primaryKey = primaryKey;
-            return this;
-        }
-
         public Builder setTimerState(final TimerState timerState) {
             this.timerState = timerState;
             return this;
